/*
 *
 * (C) 2004-2005 Denora Team
 * Contact us at info@nomadirc.net
 *
 * Please read COPYING and README for furhter details.
 *
 * Based on the original code of Anope by Anope Team.
 * Based on the original code of Thales by Lucas.
 * 
 * $Id: modules.h 402 2005-07-10 02:46:23Z trystan $
 *
 */

#ifndef MODULES_H
#define MODULES_H

/* Cross OS compatibility macros */
#ifdef _WIN32
typedef HMODULE deno_module_t;

#define deno_modopen(file)		LoadLibrary(file)
/* deno_moderr in modules.c */
#define deno_modsym(file, symbol)	(void *)GetProcAddress(file, symbol)
#define deno_modclose(file)		FreeLibrary(file) ? 0 : 1
#define deno_modclearerr()		SetLastError(0)
#define MODULE_EXT			".dll"

#else
typedef void *	deno_module_t;

#ifdef HAS_RTLD_LOCAL
#define deno_modopen(file)		dlopen(file, RTLD_LAZY|RTLD_LOCAL)
#else
#define deno_modopen(file) 		dlopen(file, RTLD_LAZY)
#endif
#define deno_moderr()			dlerror()
#define deno_modsym(file, symbol)	dlsym(file, DL_PREFIX symbol)
#define deno_modclose(file)		dlclose(file)
#define deno_modclearerr()		errno = 0
#define MODULE_EXT			".so"

#endif

typedef enum { CORE,PROTOCOL,THIRD,SUPPORTED,QATESTED } MODType;

/*************************************************************************/
#define CMD_HASH(x)      (((x)[0]&31)<<5 | ((x)[1]&31))	/* Will gen a hash from a string :) */
#define MAX_CMD_HASH 1024

#define MOD_STOP 1
#define MOD_CONT 0

#define MOD_UNIQUE 0
#define MOD_HEAD   1
#define MOD_TAIL   2

#define STATSERV OS_cmdTable
#define IRCD IRCD_cmdTable
#define MODULE_HASH Module_table
#define EVENT EVENT_cmdTable
#define EVENTHOOKS HOOK_cmdTable
#define CRONEVENT CRONHOOK_cmdTable

/**********************************************************************
 * Module Returns
 **********************************************************************/
#define MOD_ERR_OK			0
#define MOD_ERR_MEMORY		1
#define MOD_ERR_PARAMS		2
#define MOD_ERR_EXISTS		3
#define MOD_ERR_NOEXIST		4
#define MOD_ERR_NOUSER		5
#define MOD_ERR_NOLOAD      6
#define MOD_ERR_NOUNLOAD    7
#define MOD_ERR_SYNTAX		8
#define MOD_ERR_NODELETE	9
#define MOD_ERR_UNKNOWN		10
#define MOD_ERR_FILE_IO     11
#define MOD_ERR_NOSERVICE   12
#define MOD_ERR_NO_MOD_NAME 13

/*************************************************************************/
/* Macros to export the Module API functions/variables */
#ifndef _WIN32
#define MDE
#else
#ifndef MODULE_COMPILE
#define MDE __declspec(dllexport)
#else
#define MDE __declspec(dllimport)
#endif
#endif
/*************************************************************************/

/* Structure for information about a *Serv command. */

typedef struct Command_ Command;
typedef struct CommandHash_ CommandHash;
typedef struct Module_ Module;
typedef struct ModuleHash_ ModuleHash;
typedef struct Message_ Message;
typedef struct MessageHash_ MessageHash;
typedef struct ModuleCallBack_ ModuleCallBack;
typedef struct EvtMessage_ EvtMessage;
typedef struct EvtMessageHash_ EvtMessageHash;
typedef struct EvtHook_ EvtHook;
typedef struct EvtHookHash_ EvtHookHash;
typedef struct CronEvent_ CronEvent;
typedef struct CronEventHash_ CronEventHash;
typedef struct ModuleLang_ ModuleLang;

extern MDE CommandHash *STATSERV[MAX_CMD_HASH];
extern MDE MessageHash *IRCD[MAX_CMD_HASH];
extern MDE EvtMessageHash *EVENT[MAX_CMD_HASH];
extern MDE EvtHookHash *EVENTHOOKS[MAX_CMD_HASH];
extern MDE CronEventHash *CRONEVENT[MAX_CMD_HASH];
extern MDE ModuleHash *MODULE_HASH[MAX_CMD_HASH];

struct ModuleLang_ {
    int argc;
    char **argv;
};

struct Module_ {
	char *name;
	char *filename;
	void *handle;
	time_t time;
	char *version;
	char *author;
	MODType type;
	void (*operHelp)(User *u); /* 5 */

	MessageHash *msgList[MAX_CMD_HASH];
	ModuleLang lang[NUM_LANGS];
};

struct ModuleHash_ {
        char *name;
        Module *m;
        ModuleHash *next;
};

struct Command_ {
    char *name;
    int (*routine)(User *u);
    int (*has_priv)(User *u);	/* Returns 1 if user may use command, else 0 */

    int helpmsg_all;	/* Displayed to all users; -1 = no message */
    int helpmsg_reg;	/* Displayed to regular users only */
    int helpmsg_admin;	/* Displayed to Stats admins only */
	int helpmsg_oper;

    char *help_param1;
    char *help_param2;
    char *help_param3;
    char *help_param4;

    /* Module related stuff */
    int core;           /* Can this command be deleted? */
    char *mod_name;	/* Name of the module who owns us, NULL for core's  */
    char *service;	/* Service we provide this command for */
    int (*all_help)(User *u);
    int (*regular_help)(User *u);
    int (*admin_help)(User *u);

    Command *next;	/* Next command responsible for the same command */
};

struct CommandHash_ {
        char *name;	/* Name of the command */
        Command *c;	/* Actual command */
        CommandHash *next; /* Next command */
};

struct Message_ {
    char *name;
    int (*func)(char *source, int ac, char **av);
    int core;
    char *mod_name;
    Message *next;
};

struct MessageHash_ {
        char *name;
        Message *m;
        MessageHash *next;
};

struct ModuleCallBack_ {
	char *name;
	char *owner_name;
	time_t when;
	int (*func)(int argc, char *argv[]);
	int argc;
	char **argv;
	ModuleCallBack *next;
};

struct EvtMessage_ {
    char *name;
    int (*func)(char *source, int ac, char **av);
    int core;
    char *mod_name;
    EvtMessage *next;
};

struct EvtMessageHash_ {
        char *name;
        EvtMessage *evm;
        EvtMessageHash *next;
};


struct EvtHook_ {
    int (*func)(int argc, char **argv);
    int core;
	char *name;
    char *mod_name;
    EvtHook *next;
};

struct EvtHookHash_ {
        char *name;
        EvtHook *evh;
        EvtHookHash *next;
};


struct CronEvent_ {
    int (*func)(const char *name);
	char *name;
    int core;
    char *mod_name;
    CronEvent *next;
};

struct CronEventHash_ {
        char *name;
        CronEvent *evh;
        CronEventHash *next;
};

/*************************************************************************/
/* Module Managment Functions */
MDE Module *createModule(char *filename);	/* Create a new module, using the given name */
int destroyModule(Module *m);		/* Delete the module */
int addModule(Module *m);		/* Add a module to the module hash */
int delModule(Module *m);		/* Remove a module from the module hash */
MDE Module *findModule(char *name);		/* Find a module */
int loadModule(Module *m,User *u);	/* Load the given module into the program */
int unloadModule(Module *m, User *u);	/* Unload the given module from the pro */
int prepForUnload(Module *m);		/* Prepare the module for unload */
MDE void moduleAddVersion(const char *version);
MDE void moduleAddAuthor(const char *author);
void modules_init(void);
void modules_delayed_init(void);
void moduleCallBackPrepForUnload(char *mod_name);
MDE void moduleCallBackDeleteEntry(ModuleCallBack * prev);
MDE char *moduleGetLastBuffer(void);
MDE void moduleSetHelpHelp(void (*func) (User * u));
MDE void moduleDisplayHelp(int service, User *u);
int moduleCopyFile(char *name);
int displayCommand(Command * c);
int displayCommandFromHash(CommandHash * cmdTable[], char *name);
int displayMessageFromHash(char *name);
int displayMessage(Message * m);
int destroyMessageHash(MessageHash * mh);
int destroyCommandHash(CommandHash * ch);
ModuleCallBack *moduleCallBackFindEntry(char *mod_name, boolean * found);
int displayEventMessage(EvtMessage * evm);
int displayEventHook(EvtHook * evh);
int displayHookFromHash(char *name);
int displayEvtMessageFromHash(char *name);
EvtMessage *find_event(const char *name);
EvtHook *find_eventhook(const char *name);
int addEventHook(EvtHookHash * hookEvtTable[], EvtHook * evh);

MDE void moduleSetOperHelp(void (*func) (User * u));
MDE int moduleAddHelp(Command * c, int (*func) (User * u));
MDE int moduleAddRegHelp(Command * c, int (*func) (User * u));
MDE int moduleAddAdminHelp(Command * c, int (*func) (User * u));
/*************************************************************************/
/*************************************************************************/
/* Command Managment Functions */
MDE Command *createCommand(const char *name,int (*func)(User *u),int (*has_priv)(User *u),int help_all, int help_reg, int help_oper, int help_admin);
MDE int destroyCommand(Command *c);					/* destroy a command */
MDE int addCoreCommand(CommandHash *cmdTable[], Command *c);	/* Add a command to a command table */
MDE int moduleAddCommand(CommandHash *cmdTable[], Command *c, int pos);
MDE int addCommand(CommandHash *cmdTable[], Command *c,int pos);
MDE int delCommand(CommandHash *cmdTable[], Command *c,char *mod_name);		/* Del a command from a cmd table */
MDE int moduleDelCommand(CommandHash *cmdTable[],char *name);		/* Del a command from a cmd table */
Command *findCommand(CommandHash *cmdTable[], const char *name);	/* Find a command */

/*************************************************************************/

/* Message Managment Functions */
MDE Message *createMessage(const char *name,int (*func)(char *source, int ac, char **av));
Message *findMessage(MessageHash *msgTable[], const char *name);	/* Find a Message */
MDE int addMessage(MessageHash *msgTable[], Message *m, int pos);		/* Add a Message to a Message table */
MDE int addCoreMessage(MessageHash *msgTable[], Message *m);		/* Add a Message to a Message table */
MDE int moduleAddMessage(Message *m, int pos);
int delMessage(MessageHash *msgTable[], Message *m, char *mod_name);		/* Del a Message from a msg table */
MDE int moduleDelMessage(char *name);
int destroyMessage(Message *m);					/* destroy a Message*/

/*************************************************************************/
MDE int moduleAddCallback(char *name,time_t when,int (*func)(int argc, char *argv[]),int argc, char **argv);
MDE void moduleDelCallback(char *name);

MDE char *moduleGetData(ModuleData **md, char *key);			/* Get the value for this key from this struct */
MDE int moduleAddData(ModuleData **md, char *key, char *value);		/* Set the value for this key for this struct */
MDE void moduleDelData(ModuleData **md, char *key);				/* Delete this key/value pair */
MDE void moduleDelAllData(ModuleData **md);					/* Delete all key/value pairs for this module for this struct */
void moduleDelAllDataMod(Module *m);					/* remove all module data from all structs for this module */
int moduleDataDebug(ModuleData **md);					/* Allow for debug output of a moduleData struct */
MDE boolean moduleMinVersion(int major,int minor,int patch,int build);	/* Checks if the current version of denora is before or after a given verison */

MDE EvtMessage *createEventHandler(char *name, int (*func) (char *source, int ac, char **av));
EvtMessage *findEventHandler(EvtMessageHash * msgEvtTable[], const char *name);
int addCoreEventHandler(EvtMessageHash * msgEvtTable[], EvtMessage * evm);
MDE int moduleAddEventHandler(EvtMessage * evm);
MDE int moduleEventDelHandler(char *name);
int delEventHandler(EvtMessageHash * msgEvtTable[], EvtMessage * evm, char *mod_name);
int destroyEventHandler(EvtMessage * evm);
int addEventHandler(EvtMessageHash * msgEvtTable[], EvtMessage * evm);

EvtHook *createEventHook(char *name, int (*func) (int argc, char **argv));
EvtHook *findEventHook(EvtHookHash * HookEvtTable[], const char *name);
int addCoreEventHook(EvtHookHash * HookEvtTable[], EvtHook * evh);
int moduleAddEventHook(EvtHook * evh);
int moduleEventDelHook(const char *name);
int delEventHook(EvtHookHash * HookEvtTable[], EvtHook * evh, char *mod_name);
int destroyEventHook(EvtHook * evh);
extern char *mod_current_evtbuffer;
Message *first_message(void);
Message *next_message(void);

MessageHash *first_messagehash(void);
MessageHash *next_messagehash(void);

Command *first_command(void);
Command *next_command(void);

CommandHash *first_commandhash(void);
CommandHash *next_commandhash(void);

void destroy_all(void);

/*************************************************************************/

MDE CronEvent *createCronEvent(const char *name, int (*func) (const char *name));
CronEvent *findCronEvent(CronEventHash * hookEvtTable[], const char *name);
int addCronEvent(CronEventHash * hookEvtTable[], CronEvent * evh);
int delCronEvent(CronEventHash * msgEvtTable[], CronEvent * evm, char *mod_name);
int displayCronFromHash(char *name);
int displayCronEvent(CronEvent * evh);
MDE int moduleCronEventDel(const char *name);
MDE int moduleAddCronEvent(CronEvent * evh);

/*************************************************************************/

Command *lookup_cmd(Command *list, char *name);
void run_cmd(char *service, User *u, Command *list, char *name);
void help_cmd(char *service, User *u, Command *list, char *name);
void do_run_cmd(char *service, User * u, Command *c,const char *cmd);
void do_help_cmd(char *service, User * u, Command *c,const char *cmd);
MDE void mod_help_cmd(char *service, User *u, CommandHash *cmdTable[],const char *cmd);
MDE void mod_run_cmd(char *service, User *u, CommandHash *cmdTable[],const char *cmd);

void moduleAddMsgs(void);

MDE void moduleSetType(MODType type);

MDE void moduleInsertLanguage(int langNumber, int ac, char **av);
MDE void moduleNoticeLang(char *source, User *u, int number, ...);
MDE void moduleDeleteLanguage(int langNumber);

int moduleCopyCoreFile(char *name);
int loadCoreModule(Module * m, User * u);
Module *createCoreModule(char *filename);

#endif
/* EOF */
