/* StatServ core functions
 *
 * (C) 2003-2005 Anope Team
 * Contact us at info@anope.org
 *
 * Please read COPYING and README for further details.
 *
 * Based on the original code of Epona by Lara.
 * Based on the original code of Services by Andy Church. 
 * 
 * $Id: ss_exclude.c 389 2005-07-05 22:45:48Z trystan $
 *
 */
/*************************************************************************/

#include "denora.h"

static int do_exclude(User * u);
int DenoraInit(int argc, char **argv);
void DenoraFini(void);

/**
 * Create the command, and tell anope about it.
 * @param argc Argument count
 * @param argv Argument list
 * @return MOD_CONT to allow the module, MOD_STOP to stop it
 **/
int DenoraInit(int argc, char **argv)
{
    Command *c;

    if (denora->debug >= 2) {
        protocol_debug(NULL, argc, argv);
    }
    moduleAddAuthor("Denora");
    moduleAddVersion("$Id: ss_exclude.c 389 2005-07-05 22:45:48Z trystan $");
    moduleSetType(CORE);

    c = createCommand("EXCLUDE",   do_exclude,    is_stats_admin,      -1, -1,-1, STAT_HELP_EXCLUDE);
    moduleAddCommand(STATSERV, c, MOD_UNIQUE);

    return MOD_CONT;
}

/**
 * Unload the module
 **/
void DenoraFini(void)
{

}

static int do_exclude(User * u)
{
    Exclude *e;
    char *cmd1 = strtok(NULL, " ");
    char *cmd2 = strtok(NULL, " ");
    char *tmp = NULL;
    char *s = NULL;
    int count = 0, from = 0, to = 0;
    int nnicks = 0, i;
    int disp = 1;

    if (!cmd1) {
        notice_lang(s_StatServ, u, STAT_EXCLUDE_SYNTAX);
        return MOD_CONT;
    }

    if (!stricmp(cmd1, "ADD")) {
        if (!cmd2) {
            notice_lang(s_StatServ, u, STAT_EXCLUDE_SYNTAX);
            return MOD_CONT;
        }
        e = find_exclude(cmd2, NULL);
        if (!e) {
            make_exclude(cmd2);
            notice_lang(s_StatServ, u, STAT_EXCLUDE_ADDED, cmd2);
            rdb_query("DELETE FROM %s WHERE uname=\"%s\"", UStatsTable,
                      cmd2);
        } else {
            notice_lang(s_StatServ, u, STAT_EXCLUDE_ALREADY, cmd2);
        }
    } else if (!stricmp(cmd1, "DEL")) {
        if (!cmd2) {
            notice_lang(s_StatServ, u, STAT_EXCLUDE_SYNTAX);
            return MOD_CONT;
        }
        e = find_exclude(cmd2, NULL);
        if (e) {
            del_exclude(e);
            notice_lang(s_StatServ, u, STAT_EXCLUDE_DELETED, cmd2);
        } else {
            notice_lang(s_StatServ, u, STAT_EXCLUDE_NOT_FOUND, cmd2);
        }
    } else if (!stricmp(cmd1, "LIST")) {
        if (cmd2) {
            if (cmd2[0] == '#') {
                tmp = myStrGetOnlyToken((cmd2 + 1), '-', 0);    /* Read FROM out */
                if (!tmp) {
                    return MOD_CONT;
                }
                for (s = tmp; *s; s++) {
                    if (!isdigit(*s)) {
                        return MOD_CONT;
                    }
                }
                from = atoi(tmp);
                tmp = myStrGetTokenRemainder(cmd2, '-', 1);     /* Read TO out */
                if (!tmp) {
                    return MOD_CONT;
                }
                for (s = tmp; *s; s++) {
                    if (!isdigit(*s)) {
                        return MOD_CONT;
                    }
                }
                to = atoi(tmp);
                cmd2 = sstrdup("*");
            }
        }

        notice_lang(s_StatServ, u, STAT_EXCLUDE_LIST_HEADER);

        for (i = 0; i < 1024; i++) {
            for (e = exlists[i]; e; e = e->next) {
                if ((count + 1 >= from) && (count + 1 <= to)) {
                    notice(s_StatServ, u->nick, "%d %s", disp++, e->name);
                } else if (((from == 0) && (to == 0)) && (++nnicks <= 50)) {
                    notice(s_StatServ, u->nick, "%d %s", disp++, e->name);
                }
                count++;
            }
        }
    }

    return MOD_CONT;
}

