/* StatServ core functions
 *
 * (C) 2003-2005 Anope Team
 * Contact us at info@anope.org
 *
 * Please read COPYING and README for further details.
 *
 * Based on the original code of Epona by Lara.
 * Based on the original code of Services by Andy Church. 
 * 
 * $Id: ss_stats.c 389 2005-07-05 22:45:48Z trystan $
 *
 */
/*************************************************************************/

#include "denora.h"

static int do_stats(User * u);
int DenoraInit(int argc, char **argv);
void DenoraFini(void);

/**
 * Create the command, and tell anope about it.
 * @param argc Argument count
 * @param argv Argument list
 * @return MOD_CONT to allow the module, MOD_STOP to stop it
 **/
int DenoraInit(int argc, char **argv)
{
    Command *c;

    if (denora->debug >= 2) {
        protocol_debug(NULL, argc, argv);
    }
    moduleAddAuthor("Denora");
    moduleAddVersion("$Id: ss_stats.c 389 2005-07-05 22:45:48Z trystan $");
    moduleSetType(CORE);

    c = createCommand("STATS",     do_stats,      is_oper,             -1, -1,-1, STAT_HELP_STATS);
    moduleAddCommand(STATSERV, c, MOD_UNIQUE);

    return MOD_CONT;
}

/**
 * Unload the module
 **/
void DenoraFini(void)
{

}

/* STATS command. */

static int do_stats(User * u)
{
    char *what = strtok(NULL, "");

    SET_SEGV_LOCATION();

    if (!what) {
        notice_lang(s_StatServ, u, STAT_STATS_SERVER_COUNT, NetworkName,
                    stats->servers, stats->servers_max);
        notice_user(s_StatServ, u, " ");
        notice_lang(s_StatServ, u, STAT_STATS_CHAN_COUNT, NetworkName,
                    stats->chans, stats->chans_max);
        notice_user(s_StatServ, u, " ");
        notice_lang(s_StatServ, u, STAT_STATS_USER_COUNT, NetworkName,
                    stats->users, stats->users_max);
    } else if (!stricmp(what, "SERVERS")) {
        notice_lang(s_StatServ, u, STAT_STATS_SERVER_COUNT, NetworkName,
                    stats->servers, stats->servers_max);
    } else if (!stricmp(what, "CHANNELS")) {
        notice_lang(s_StatServ, u, STAT_STATS_CHAN_COUNT, NetworkName,
                    stats->chans, stats->chans_max);
    } else if (!stricmp(what, "USERS")) {
        notice_lang(s_StatServ, u, STAT_STATS_USER_COUNT, NetworkName,
                    stats->users, stats->users_max);
    } else {
        syntax_error(s_StatServ, u, "STATS", STAT_STATS_SYNTAX);
    }

    return MOD_CONT;
}

