/* StatServ core functions
 *
 * (C) 2003-2005 Anope Team
 * Contact us at info@anope.org
 *
 * Please read COPYING and README for further details.
 *
 * Based on the original code of Epona by Lara.
 * Based on the original code of Services by Andy Church. 
 * 
 * $Id: ss_userdebug.c 408 2005-07-11 00:03:44Z trystan $
 *
 */
/*************************************************************************/

#include "denora.h"

static int do_userdebug(User * u);
int DenoraInit(int argc, char **argv);
void DenoraFini(void);

/**
 * Create the command, and tell anope about it.
 * @param argc Argument count
 * @param argv Argument list
 * @return MOD_CONT to allow the module, MOD_STOP to stop it
 **/
int DenoraInit(int argc, char **argv)
{
    Command *c;

    if (denora->debug >= 2) {
        protocol_debug(NULL, argc, argv);
    }
    moduleAddAuthor("Denora");
    moduleAddVersion("$Id: ss_userdebug.c 408 2005-07-11 00:03:44Z trystan $");
    moduleSetType(CORE);

    c = createCommand("USERDEBUG", do_userdebug,  is_stats_admin,      -1, -1,-1, STAT_HELP_USERDEBUG); 
    moduleAddCommand(STATSERV, c, MOD_UNIQUE);

    return MOD_CONT;
}

/**
 * Unload the module
 **/
void DenoraFini(void)
{

}

static int do_userdebug(User * u)
{
    User *u2;
    char *temp = moduleGetLastBuffer();
    char *nick = myStrGetTokenRemainder(temp, ' ', 0);
    struct tm tm;
    char buf[BUFSIZE];

    notice(s_StatServ, u->nick,
           "============== USER DEBUG ==============");

    if (!nick) {
        denora_free(temp);
        return MOD_CONT;
    }

    if ((u2 = finduser(nick))) {
        notice(s_StatServ, u->nick, "User: %s!%s@%s", u2->nick,
               u2->username, u2->host);
        if (ircd->vident && ircd->vhost) {
            if (u2->vident) {
                alog(LOG_NORMAL, "Vhost: %s@%s", u2->vident, u2->vhost);
            } else {
                alog(LOG_NORMAL, "Vhost: %s", u2->vhost);
            }
        } else {
            alog(LOG_NORMAL, "Vhost: %s", u2->vhost);
        }

        alog(LOG_NORMAL, "Host Resolves to %s", host_resolve(u2->host));

        tm = *localtime(&u2->my_signon);
        strftime(buf, sizeof(buf) - 1, MYTIME, &tm);
        alog(LOG_NORMAL, "Sign On At : %s", buf);

        tm = *localtime(&u2->timestamp);
        strftime(buf, sizeof(buf) - 1, MYTIME, &tm);
        alog(LOG_NORMAL, "Time Stamp of Nick :  %s", buf);

        alog(LOG_NORMAL, "Server : %s", u2->server->name);

    } else {
        alog(LOG_NORMAL, "%s user was not found in the user struct", nick);
    }

    denora_free(temp);
    denora_free(nick);

    return MOD_CONT;
}

