/*
 *
 * (C) 2004-2005 Denora Team
 * Contact us at info@nomadirc.net
 *
 * Please read COPYING and README for furhter details.
 *
 * Based on the original code of Anope by Anope Team.
 * Based on the original code of Thales by Lucas.
 * 
 * $Id: html.c 439 2005-07-27 17:16:46Z drstein $
 *
 */

#include "denora.h"

void get_map(FILE * ptr, int level);
FILE *html_open(char *filename);
FILE *html_open_write(char *filename);
void html_close(FILE * ptr);
char *html_buffer_file(FILE * ptr);
void html_current_channels(FILE * ptr);
void html_title(FILE * ptr);
void html_kicks_title(FILE * ptr);
void html_server_table(FILE * ptr);
void html_credits(FILE * ptr);
void html_server_details(FILE * ptr);
void html_dailystats(FILE * ptr);
void html_lang(FILE * ptr);
void html_top10_topics_title(FILE * ptr);
void html_ctcp_table(FILE * ptr);
void html_top10chan(FILE * ptr);
void html_top10kicks(FILE * ptr);
void html_version(FILE * ptr);
void html_tld_table(FILE * ptr);
void html_top10_topics(FILE * ptr);
void html_tld_title(FILE * ptr);
void html_netstats(FILE * ptr);
void html_curnetstats_title(FILE * ptr);
void html_netmap_title(FILE * ptr);
void html_top10client_title(FILE * ptr);
void html_popularchan_title(FILE * ptr);
void html_mosttopic_title(FILE * ptr);
void html_netstats_title(FILE * ptr);
void html_serverdetails_title(FILE * ptr);
void html_top10_chanever_title(FILE * ptr);
void html_serverlist_title(FILE * ptr);
void html_top10_onlinechan_title(FILE * ptr);
void html_dailystats_title(FILE * ptr);
void html_mostkick_title(FILE * ptr);

/*************************************************************************/

FILE *html_open(char *filename)
{
    FILE *htmlfile;

    SET_SEGV_LOCATION();

    if ((htmlfile = fopen(filename, "r")) == NULL) {
        alog(LOG_ERROR, "Error accessing file: %s", filename);
        return NULL;
    }
    return htmlfile;
}

/*************************************************************************/

FILE *html_open_write(char *filename)
{
    FILE *htmlfile;

    SET_SEGV_LOCATION();

    if (filename) {
        if ((htmlfile = fopen(filename, "w")) == NULL) {
            alog(LOG_ERROR, langstring(ALOG_HTML_ERR_WRITE), filename);
            return NULL;
        }
        return htmlfile;
    }
    return NULL;
}

/*************************************************************************/

void html_close(FILE * ptr)
{
    SET_SEGV_LOCATION();
    fclose(ptr);
}

/*************************************************************************/

char *html_buffer_file(FILE * ptr)
{
    char buf[BUFSIZE];
    *buf = '\0';

    SET_SEGV_LOCATION();

    while (fgets(buf, BUFSIZE - 1, ptr)) {
        buf[strlen(buf) - 1] = 0;
    }
    return sstrdup(buf);
}

/*************************************************************************/

void html_tld_table(FILE * ptr)
{
    TLD *t;
    char *tempcc;
    char *tempc;

    if (ptr) {
        fprintf(ptr, langstr(HTML_TLD_HEADER));
        t = tld_sort();
        while (t) {
            tempcc = char_encode(t->countrycode);
            tempc = char_encode(t->country);
            fprintf(ptr, langstr(HTML_TLD_CONTENT),
                    tempcc, tempc, t->count, t->overall);
            denora_free(tempcc);
            denora_free(tempc);
            t = t->nextsort;
        }
        fprintf(ptr, langstr(HTML_TLD_FOOTER));
    }
}

/*************************************************************************/

void html_current_channels(FILE * ptr)
{
    StatsChannel *s;
    char *tempc;
    Exclude *e;

    SET_SEGV_LOCATION();

    if (ptr) {
        fprintf(ptr, getstring(NULL, HTML_CHANNEL_HEADER));
        s = top10_sort();
        while (s) {
            if (s->in_use) {
                e = find_exclude(s->name, NULL);
                if (!e) {
                    tempc = char_encode(s->name);
                    fprintf(ptr, langstring(HTML_CHANNEL_CONTENT),
                            tempc, s->usercount);
                    denora_free(tempc);
                }
            }
            s = s->nextsort;
        }
        fprintf(ptr, getstring(NULL, HTML_CHANNEL_FOOTER));
        SET_SEGV_LOCATION();
    }
}

/*************************************************************************/

void html_ctcp_table(FILE * ptr)
{
    CTCPVerStats *s = NULL;
    char *tempc;
    SET_SEGV_LOCATION();

    if (ptr) {
        fprintf(ptr, getstring(NULL, HTML_CTCP_HEADER));
        SET_SEGV_LOCATION();
        s = ctcp_sort();
        while (s) {
            tempc = char_encode(s->version);
            fprintf(ptr, getstring(NULL, HTML_CTCP_CONTENT),
                    tempc, s->overall);
            s = s->nextsort;
            denora_free(tempc);
        }
        fprintf(ptr, getstring(NULL, HTML_CTCP_FOOTER));
        SET_SEGV_LOCATION();
    }
}

/*************************************************************************/

void html_top10chan(FILE * ptr)
{
    StatsChannel *s;
    char *tempc;
    Exclude *e;
    SET_SEGV_LOCATION();

    if (ptr) {
        fprintf(ptr, langstring(HTML_CHAN_TOP10_HEADER));
        SET_SEGV_LOCATION();
        s = totaljoin_sort();
        while (s) {
            if (s->joincounter) {
                e = find_exclude(s->name, NULL);
                if (!e) {
                    tempc = char_encode(s->name);
                    if (s->in_use) {
                        fprintf(ptr, langstring(HTML_CHAN_TOP10_CONTENT),
                                tempc, s->joincounter);
                    } else {
                        fprintf(ptr, langstring(HTML_CHAN_TOP10_OFFLINE),
                                tempc, s->joincounter);
                    }
                    denora_free(tempc);
                }
            }
            s = s->nextsort;
        }
        fprintf(ptr, langstring(HTML_CHAN_TOP10_FOOTER));
        SET_SEGV_LOCATION();
    }
}

/*************************************************************************/

void html_top10kicks(FILE * ptr)
{
    StatsChannel *s;
    char *tempc;
    Exclude *e;
    SET_SEGV_LOCATION();

    if (ptr) {
        fprintf(ptr, langstring(HTML_CHAN_TOP10_KICKS_HEADER));
        SET_SEGV_LOCATION();
        s = kick_sort();
        while (s) {
            if (s->kickcount) {
                e = find_exclude(s->name, NULL);
                if (!e) {
                    tempc = char_encode(s->name);
                    if (s->in_use) {
                        fprintf(ptr,
                                langstring
                                (HTML_CHAN_TOP10_KICKS_CONTENT_IN_USE),
                                tempc, s->kickcount);
                    } else {
                        fprintf(ptr,
                                langstring(HTML_CHAN_TOP10_KICKS_CONTENT),
                                tempc, s->kickcount);
                    }
                    denora_free(tempc);
                }
            }
            s = s->nextsort;
        }
        fprintf(ptr, langstring(HTML_CHAN_TOP10_KICKS_FOOTER));
        SET_SEGV_LOCATION();
    }
}

/*************************************************************************/

void html_top10_topics(FILE * ptr)
{
    StatsChannel *s;
    char *tempc;
    Exclude *e;
    SET_SEGV_LOCATION();

    if (ptr) {
        fprintf(ptr, langstring(HTML_TOP10_TOPIC_HEADER));
        SET_SEGV_LOCATION();
        s = topic_sort();
        while (s) {
            if (s->topic_count) {
                e = find_exclude(s->name, NULL);
                if (!e) {
                    tempc = char_encode(s->name);
                    if (s->in_use) {
                        fprintf(ptr,
                                langstring
                                (HTML_CHAN_TOP10_KICKS_CONTENT_IN_USE),
                                tempc, s->topic_count);
                    } else {
                        fprintf(ptr,
                                langstring(HTML_CHAN_TOP10_KICKS_CONTENT),
                                tempc, s->topic_count);
                    }
                    denora_free(tempc);
                }
            }
            s = s->nextsort;
        }
        fprintf(ptr, langstring(HTML_TOP10_TOPIC_FOOTER));
        SET_SEGV_LOCATION();
    }
}

/*************************************************************************/

void html_server_table(FILE * ptr)
{
    ServStats *s, *next;
    char *tempc;
    SET_SEGV_LOCATION();

    if (ptr) {
        fprintf(ptr, getstring(NULL, HTML_SERVER_HEADER));
        SET_SEGV_LOCATION();
        s = first_statsserver();
        while (s) {
            next = next_statserver();
            tempc = char_encode(s->name);
            if (s->split_stats) {
                fprintf(ptr, getstring(NULL, HTML_SERVER_CONTENT),
                        s->name, tempc);
            } else {
                fprintf(ptr,
                        getstring(NULL, HTML_SERVER_CONTENT_ONLINE),
                        s->name, tempc);
            }
            denora_free(tempc);
            s = next;
        }
        fprintf(ptr, getstring(NULL, HTML_SERVER_FOOTER));
        SET_SEGV_LOCATION();
    }
}

/*************************************************************************/

void html_credits(FILE * ptr)
{
    struct tm tm;
    char timebuf[64];
    time_t ts;
    char *tempc;
    ts = time(NULL);

    tempc = char_encode(ServerName);
    fprintf(ptr, "StatServ Information:<br>");
    fprintf(ptr, "%s compiled on [%s %s %s]<br>", tempc,
            denora->date, denora->time, denora->build);
    fprintf(ptr,
            "<a href=http://denora.nomadirc.net>denora.nomadirc.net</a><br>");
    tm = *localtime(&ts);
    strftime_lang(timebuf, sizeof(timebuf), NULL,
                  STRFTIME_DATE_TIME_FORMAT, &tm);
    fprintf(ptr, "HTML Last Generated at %s", timebuf);
}

/*************************************************************************/

void html_server_details(FILE * ptr)
{
    ServStats *s, *next;
    Server *serv;
    char timebuf[64];
    struct tm tm;

    SET_SEGV_LOCATION();

    if (ptr) {
        SET_SEGV_LOCATION();
        s = first_statsserver();
        while (s) {
            serv = findserver(s->name);
            next = next_statserver();

            fprintf(ptr, getstring(NULL, HTML_SERVER_DETAIL_HEADER),
                    s->name, s->name);
            if (s->split_stats) {
                if (s->lastseen) {
                    tm = *localtime(&s->lastseen);
                    strftime_lang(timebuf, sizeof(timebuf), NULL,
                                  STRFTIME_DATE_TIME_FORMAT, &tm);
                    fprintf(ptr,
                            langstring(HTML_SERVER_DETAIL_LAST_SEEN),
                            timebuf);
                }
            } else {
                tm = *localtime(&s->maxusertime);
                strftime_lang(timebuf, sizeof(timebuf), NULL,
                              STRFTIME_DATE_TIME_FORMAT, &tm);
                fprintf(ptr, langstring(HTML_SERVER_DETAIL_USERS),
                        s->currentusers, 0, s->maxusers, timebuf);

                tm = *localtime(&s->maxopertime);
                strftime_lang(timebuf, sizeof(timebuf), NULL,
                              STRFTIME_DATE_TIME_FORMAT, &tm);
                fprintf(ptr, langstring(HTML_SERVER_DETAIL_OPERS),
                        s->opers, 0, s->maxopers, timebuf);
            }
            fprintf(ptr, langstring(HTML_SERVER_DETAIL_TOTALUSERSEVER),
                    s->totalusersever);

            fprintf(ptr, langstring(HTML_SERVER_DETAIL_IRCOPKILLS),
                    s->ircopskills);
            fprintf(ptr, langstring(HTML_SERVER_DETAIL_SERVERKILLS),
                    s->serverkills);
            if (s->highestping) {
                tm = *localtime(&s->maxpingtime);
                strftime_lang(timebuf, sizeof(timebuf), NULL,
                              STRFTIME_DATE_TIME_FORMAT, &tm);
                fprintf(ptr,
                        langstring(HTML_SERVER_DETAIL_HIGHESTPING),
                        s->highestping, timebuf);
            }
            if (!s->split_stats) {
                if (serv) {
                    fprintf(ptr,
                            langstring(HTML_SERVER_DETAIL_CURPING),
                            serv->ping);
                }
            }
            fprintf(ptr, langstring(HTML_SERVER_DETAIL_SPLITS),
                    (s ? s->splits : 0));
            fprintf(ptr, langstring(HTML_SERVER_DETAIL_FOOTER));

            s = next;
        }
        SET_SEGV_LOCATION();
    }
}

/*************************************************************************/

void html_version(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstr(HTML_VERSION), denora->version, ircd->name);
    }
}

/*************************************************************************/

void html_mostkick_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstr(HTML_MOST_UNWELCOME_TITLE));
    }
}

/*************************************************************************/

void html_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, getstring(NULL, HTML_TITLE), NetworkName);
    }
}

/*************************************************************************/

void get_map(FILE * ptr, int level)
{
    Server *s;
    if (!level && !uplink) {
        s = findserver(ServerName);
        fprintf(ptr,
                "<table border=0><tr><th>Server Name</th><th>Users/Max</th><th>Opers/Max</th><th>Lag/Max</th></tr>");
        fprintf(ptr,
                "<tr><td>%s</td><td>%ld/%ld</td><td>%ld/%ld</td><td>%ld/%ld</td></tr>\n",
                s->name, (long int) s->ss->currentusers,
                (long int) s->ss->maxusers, (long int) s->ss->opers,
                (long int) s->ss->maxopers, (long int) s->ping,
                (long int) s->ss->highestping);
        fprintf(ptr, "</table>");
    }
}

/*************************************************************************/

void html_kicks_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_CHAN_TOP10_KICKS_TITLE));
    }
}

/*************************************************************************/

void html_tld_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_TLD_TITLE));
    }
}

/*************************************************************************/

void html_lang(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_LANG));
    }
}

/*************************************************************************/

void html_top10_topics_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_TOP10_TOPIC_TITLE));
    }
}

/*************************************************************************/

void html_curnetstats_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_CUR_NETSTATS_TITLE));
    }
}

/*************************************************************************/

void html_dailystats_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_DAILY_TITLE));
    }
}

/*************************************************************************/

void html_top10_onlinechan_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstr(HTML_TOP10_ONLINE_CHANS_TITLE));
    }
}

/*************************************************************************/

void html_serverlist_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstr(HTML_SERVERLIST_TITLE));
    }
}

/*************************************************************************/

void html_top10_chanever_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_TOP10_CHANEVER_TITLE));
    }
}

/*************************************************************************/

void html_serverdetails_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_SERVERDETAILS_TITLE));
    }
}

/*************************************************************************/

void html_top10client_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_TOP10_CLIENT_TITLE));
    }
}

/*************************************************************************/

void html_netmap_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_NETMAP_TITLE));
    }
}

/*************************************************************************/

void html_popularchan_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_POPULAR_TITLE));
    }
}

/*************************************************************************/

void html_mosttopic_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_MOST_TOPIC_TITLE));
    }
}

/*************************************************************************/

void html_netstats_title(FILE * ptr)
{
    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, langstring(HTML_NETSTATS_TITLE));
    }
}

/*************************************************************************/

void html_dailystats(FILE * ptr)
{
    char timebuf[64];
    struct tm tm;

/*
<tr><td colspan="2">Total Users Connected:</td>
<td colspan="2"> 14839</td></tr>
*/

    SET_SEGV_LOCATION();
    if (ptr) {
        fprintf(ptr, getstring(NULL, HTML_DAILY_HEADER));
        tm = *localtime(&stats->daily_users_time);
        strftime_lang(timebuf, sizeof(timebuf), NULL,
                      STRFTIME_DATE_TIME_FORMAT, &tm);
        fprintf(ptr, getstring(NULL, HTML_DAILY_USER), stats->daily_users,
                timebuf);
        fprintf(ptr, getstring(NULL, HTML_DAILY_CHAN), stats->daily_chans);
        tm = *localtime(&stats->daily_opers_time);
        strftime_lang(timebuf, sizeof(timebuf), NULL,
                      STRFTIME_DATE_TIME_FORMAT, &tm);
        fprintf(ptr, getstring(NULL, HTML_DAILY_OPERS), stats->daily_opers,
                timebuf);
        tm = *localtime(&stats->daily_servers_time);
        strftime_lang(timebuf, sizeof(timebuf), NULL,
                      STRFTIME_DATE_TIME_FORMAT, &tm);
        fprintf(ptr, getstring(NULL, HTML_DAILY_SERVERS),
                stats->daily_servers, timebuf);
        fprintf(ptr, getstring(NULL, HTML_DAILY_RESET_AT));
        fprintf(ptr, getstring(NULL, HTML_DAILY_FOOTER));
    }
}

/*************************************************************************/

void html_netstats(FILE * ptr)
{
/* <tr><td colspan=2>Total Users Ever Connected</td><td colspan=2>197231</td></tr> */

    char timebuf[64];
    struct tm tm;

    SET_SEGV_LOCATION();

    if (ptr) {
        SET_SEGV_LOCATION();
        fprintf(ptr, getstring(NULL, HTML_NETSTATS_HEADER));
        tm = *localtime(&stats->users_max_time);
        strftime_lang(timebuf, sizeof(timebuf), NULL,
                      STRFTIME_DATE_TIME_FORMAT, &tm);
        fprintf(ptr, getstring(NULL, HTML_NETSTATS_USERS), stats->users,
                stats->users_max, timebuf);
        SET_SEGV_LOCATION();
        tm = *localtime(&stats->chans_max_time);
        strftime_lang(timebuf, sizeof(timebuf), NULL,
                      STRFTIME_DATE_TIME_FORMAT, &tm);
        fprintf(ptr, getstring(NULL, HTML_NETSTATS_CHANS), stats->chans,
                stats->chans_max, timebuf);
        tm = *localtime(&stats->opers_max_time);
        SET_SEGV_LOCATION();
        strftime_lang(timebuf, sizeof(timebuf), NULL,
                      STRFTIME_DATE_TIME_FORMAT, &tm);
        fprintf(ptr, getstring(NULL, HTML_NETSTATS_OPERS), stats->opers,
                stats->opers_max, timebuf);
        tm = *localtime(&stats->servers_max_time);
        strftime_lang(timebuf, sizeof(timebuf), NULL,
                      STRFTIME_DATE_TIME_FORMAT, &tm);
        fprintf(ptr, getstring(NULL, HTML_NETSTATS_SERVERS),
                stats->servers, stats->servers_max, timebuf);
        fprintf(ptr, getstring(NULL, HTML_NETSTATS_AWAY), stats->away);
        fprintf(ptr, getstring(NULL, HTML_NETSTATS_FOOTER));
    }
}

/*************************************************************************/

void do_html()
{
    char *buf, *buf1;
    char *bufold, *bufold1;
    char *html_template;
    const char *template;
    char startstr = 0;
    FILE *tpl, *opf;
#ifdef _WIN32
    char buffer[_MAX_PATH];
    char win32filename[MAXPATHLEN];

    /* Get the current working directory: */
    if (_getcwd(buffer, _MAX_PATH) == NULL) {
        alog(LOG_DEBUG, "debug: Unable to set Current working directory");
    }
#endif

    SET_SEGV_LOCATION();

#ifndef _WIN32
    template = STATS_DIR "/index.tpl";
#else
    snprintf(win32filename, sizeof(win32filename), "%s\\%s", buffer,
             "index.tpl");
    template = win32filename;
#endif
    html_template = sstrdup(template);

    if (!denora->do_html) {
        return;
    }

    if (denora->do_html) {
        if (!HTMLFilename) {
            alog(LOG_ERROR,
                 "Can't do HTML Writout as html path is not defined");
            return;
        }
    } else {
        return;
    }
    SET_SEGV_LOCATION();

    tpl = html_open(html_template);
    if (!tpl) {
        return;
    }
    opf = html_open_write(HTMLFilename);
    if (!opf) {
        denora->do_html = 0;
        return;
    }
    chmod(HTMLFilename, 0644);
    buf = malloc(STARTBUFSIZE * 2);
    bufold = buf;
    buf1 = malloc(STARTBUFSIZE * 2);
    bufold1 = buf1;
    SET_SEGV_LOCATION();

    while (fgets(buf, STARTBUFSIZE, tpl)) {

        buf1 = strstr(buf, "!TLDMAP!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_tld_table(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!TLDMAP!");
        }
        buf1 = strstr(buf, "!VERSION!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_version(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!VERSION!");
        }
        buf1 = strstr(buf, "!TOP10CHAN!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_top10chan(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!TOP10CHAN!");
        }
        buf1 = strstr(buf, "!MAP!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            get_map(opf, 0);
            fputs("</TABLE>\n", opf);
            buf = buf1 + strlen("!MAP!");
        }
        buf1 = strstr(buf, "!TITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!TITLE!");
        }
        buf1 = strstr(buf, "!NETSTATS!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_netstats(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!NETSTATS!");
        }
        buf1 = strstr(buf, "!CLIENTSTATS!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_ctcp_table(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!CLIENTSTATS!");
        }
        buf1 = strstr(buf, "!DAILYSTATS!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_dailystats(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!DAILYSTATS!");
        }
        buf1 = strstr(buf, "!SRVLIST!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_server_table(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!SRVLIST!");
        }
        buf1 = strstr(buf, "!SRVLISTDET!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_server_details(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!SRVLISTDET!");
        }
        buf1 = strstr(buf, "!CREDITS!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_credits(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!CREDITS!");
        }
        buf1 = strstr(buf, "!DAILYTOPCHAN!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_current_channels(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!DAILYTOPCHAN!");
        }
        buf1 = strstr(buf, "!TOP10KICKS!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_top10kicks(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!TOP10KICKS!");
        }
        buf1 = strstr(buf, "!TOP10KICKSTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_kicks_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!TOP10KICKSTITLE!");
        }
        buf1 = strstr(buf, "!TLDTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_tld_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!TLDTITLE!");
        }
        buf1 = strstr(buf, "!HTMLLANG!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_lang(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!HTMLLANG!");
        }
        buf1 = strstr(buf, "!TOP10TOPICS!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_top10_topics(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!TOP10TOPICS!");
        }
        buf1 = strstr(buf, "!TOP10TOPICSTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_top10_topics_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!TOP10TOPICSTITLE!");
        }
        buf1 = strstr(buf, "!CURNETSTATSTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_curnetstats_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!CURNETSTATSTITLE!");
        }
        buf1 = strstr(buf, "!DAILYSTATSTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_dailystats_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!DAILYSTATSTITLE!");
        }
        buf1 = strstr(buf, "!TOP10ONLINECHANTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_top10_onlinechan_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!TOP10ONLINECHANTITLE!");
        }
        buf1 = strstr(buf, "!SERVERLISTTILE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_serverlist_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!SERVERLISTTILE!");
        }
        buf1 = strstr(buf, "!TOP10CHANEVERTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_top10_chanever_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!TOP10CHANEVERTITLE!");
        }
        buf1 = strstr(buf, "!SERVERDETAILSTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_serverdetails_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!SERVERDETAILSTITLE!");
        }
        buf1 = strstr(buf, "!TOP10CLIENTVERTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_top10client_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!TOP10CLIENTVERTITLE!");
        }
        buf1 = strstr(buf, "!NETMAPTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_netmap_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!NETMAPTITLE!");
        }
        buf1 = strstr(buf, "!MOSTKICKTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_mostkick_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!MOSTKICKTITLE!");
        }
        buf1 = strstr(buf, "!POPULARCHANTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_popularchan_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!POPULARCHANTITLE!");
        }
        buf1 = strstr(buf, "!MOSTTOPICTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_mosttopic_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!MOSTTOPICTITLE!");
        }
        buf1 = strstr(buf, "!NETSTATSTITLE!");
        if (buf1) {
            startstr = strlen(buf) - strlen(buf1);
            fwrite(buf, startstr, 1, opf);
            html_netstats_title(opf);
            SET_SEGV_LOCATION();
            buf = buf1 + strlen("!NETSTATSTITLE!");
        }


        fputs(buf, opf);
    }
    fclose(tpl);
    fclose(opf);
    SET_SEGV_LOCATION();
    denora_free(bufold);
    denora_free(bufold1);
    denora_free(html_template);
}

/*************************************************************************/
