
/*
 *
 * (C) 2004-2005 Denora Team
 * Contact us at info@nomadirc.net
 *
 * Please read COPYING and README for furhter details.
 *
 * Based on the original code of Anope by Anope Team.
 * Based on the original code of Thales by Lucas.
 * 
 * $Id: mysql.c 421 2005-07-17 09:03:20Z trystan $
 *
 */
#include "denora.h"

#ifdef USE_MYSQL

/*************************************************************************/

/* Database Global Variables */
MYSQL *mysql;                   /* MySQL Handler */
MYSQL_FIELD *mysql_fields;      /* MySQL Fields  */
MYSQL_ROW mysql_row;            /* MySQL Row     */

void db_mysql_error(int severity, const char *msg);

/*************************************************************************/

void db_mysql_error(int severity, const char *msg)
{
    static char buf[512];

    SET_SEGV_LOCATION();

    if (mysql_error(mysql)) {
        snprintf(buf, sizeof(buf), "MySQL %s %s: %s", msg,
                 severity == SQL_WARNING ? "warning" : "error",
                 mysql_error(mysql));
    } else {
        snprintf(buf, sizeof(buf), "MySQL %s %s", msg,
                 severity == SQL_WARNING ? "warning" : "error");
    }

    log_perror(buf);

}

/*************************************************************************/

int db_mysql_init()
{

	SET_SEGV_LOCATION();

    /* If the host is not defined, assume we don't want MySQL */
    if (!SqlHost) {
        denora->do_sql = 0;
        alog(LOG_NORMAL, langstring(ALOG_SQL_DISABLED));
	    return 0;
    } else {
        denora->do_sql = 1;
        alog(LOG_NORMAL, langstring(ALOG_SQL_ENABLED), "MySQL");
        alog(LOG_SQLDEBUG, "MySQL: client version %s.",mysql_get_client_info());
    }

	SET_SEGV_LOCATION();

    /* The following configuration options are required.
     * If missing disable MySQL to avoid any problems.   
     */

    if ((denora->do_sql) && (!SqlName || !SqlUser)) {
        denora->do_sql = 0;
        alog(LOG_ERROR, langstring(ALOG_SQL_NOTSET));
	    return 0;
    }

	SET_SEGV_LOCATION();

    if (!db_mysql_open()) {
        denora->do_sql = 0;
	    return 0;
    }
	SET_SEGV_LOCATION();

    return 1;
}

/*************************************************************************/

int db_mysql_open()
{
    /* If MySQL is disabled, return 0 */
    if (!denora->do_sql)
        return 0;

	SET_SEGV_LOCATION();

    mysql = mysql_init(NULL);
    if (mysql == NULL)
        db_mysql_error(SQL_WARNING, "Unable to create mysql object");

    if (!SqlPort)
        SqlPort = 3306;

	SET_SEGV_LOCATION();

    if (SqlSock) {
        if ((!mysql_real_connect
             (mysql, SqlHost, SqlUser, (SqlPass ? SqlPass : ""), SqlName, SqlPort,
              SqlSock, 0))) {
            log_perror("MySQL Error: Cant connect to MySQL: %s\n", mysql_error(mysql));
            denora->do_sql = 0;
            return 0;
        }
    } else {
        if ((!mysql_real_connect
             (mysql, SqlHost, SqlUser, (SqlPass ? SqlPass : ""), SqlName, SqlPort,
              NULL, 0))) {
            log_perror("MySQL Error: Cant connect to MySQL: %s\n", mysql_error(mysql));
            denora->do_sql = 0;
            return 0;
        }
    }

    return 1;

}

/*************************************************************************/

int db_mysql_query(char *sql)
{
    int result, lcv;

    if (!denora->do_sql) {
        return -1;
    }

    alog(LOG_SQLDEBUG, "sql debug: %s", sql);

    mysql_ping(mysql);
    result = mysql_query(mysql, sql);

    if (result) {
        switch (mysql_errno(mysql)) {
        case CR_SERVER_GONE_ERROR:
        case CR_SERVER_LOST:

            for (lcv = 0; lcv < SqlRetries; lcv++) {
                if (db_mysql_open()) {
                    result = mysql_query(mysql, sql);
                    return (result);
                }
                sleep(SqlRetryGap);
            }

            /* If we get here, we could not connect. */
            log_perror("Unable to reconnect to database: %s\n",
                       mysql_error(mysql));
            db_mysql_error(SQL_ERROR, "connect");
            alog(LOG_NORMAL, "Disabling MYSQL due to problem with server");
            denora->do_sql = 0;
            /* Never reached. */
            break;
		case CR_COMMANDS_OUT_OF_SYNC:
			db_mysql_error(SQL_ERROR, "Commands out of sync");
			break;	    
		case ER_PARSE_ERROR:
			db_mysql_error(SQL_ERROR, "parser error");
			break;
		case ER_WRONG_VALUE_COUNT_ON_ROW:
			db_mysql_error(SQL_ERROR, "query error");
			break;
		case ER_BAD_FIELD_ERROR:
			db_mysql_error(SQL_ERROR, "Unknown field");
			break;
		case ER_NO_SUCH_TABLE:
			db_mysql_error(SQL_ERROR, "Table Doesn't exist");
			break;
		case ER_DUP_FIELDNAME:
			break;
        	case ER_BAD_NULL_ERROR:
			db_mysql_error(SQL_ERROR, "Column can not be NULL!! - check statement");
			break;
	        default:
			alog(LOG_ERROR, "Error %d",mysql_errno(mysql));
            /* Unhandled error. */
            return (result);
        }
    }

    return (0);

}

/*************************************************************************/

char *db_mysql_quote(char *sql)
{
    int slen;
    char *quoted;

    if (!sql) {
        return sstrdup("");
    }

    slen = strlen(sql);
    quoted = smalloc((1 + (slen * 2)) * sizeof(char));

    mysql_real_escape_string(mysql, quoted, sql, slen);
    return quoted;

}

/*************************************************************************/

/* I don't like using res here, maybe we can pass it as a param? */
int db_mysql_close()
{
    mysql_close(mysql);
    return 1;
}

/*************************************************************************/

#endif


